=begin
Inputbox.rb Copyright 2008 jim.foltz@gmail.com

The Inputbox class makes it easy to create user input 
dialogs by providing a consistent interface to UI.inputbox for
text fields and drop-down selection fields.

THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

=end

require 'sketchup'

# User input box class.
#
class Inputbox

  #
  # Inputbox.new( title )
  # Inputbox.new( title, options )
  #
  #   Creates a new Inputbox using the givien title.
  #   There are no options at this time.
  #
  def initialize(title, opts = {} )
    @title = title
    @prompts = []
    @choices = []
    @values = []
    @defaults = []
  end

  #
  # add( prompt [, choices, default] )
  #
  # Adds the prompt to the inputbox. With no other arguments, defaults to empty text input.
  #
  # Examples: 
  # add "Prompt 1"                            # empty text entry
  # add "Prompt 2", 2.2                       # text entry with default value
  # add "Prompt 3", [10, 12, 14, 16]          # Creates a drop-down selection from the array.
  # add "Prompt 4", %w(One Two Three)         # Defaults to first in list.
  # add "Prompt 5", %w(Four Five Six), "Five" # dropdown, 3 choices, "Five" is default
  # 
  def add(*args)
    case args.length
    when 0
      return UI.messagebox("You need at least one argument.")
    when 1
      add_prompt(args[0])
    when 2
      if args[1].is_a? Array
	add_dropdown(args[0], args[1])
      else
	add_prompt(args[0], args[1])
      end
    when 3
      add_dropdown(args[0], args[1], args[2])
    end
  end

  #
  # show
  #
  #   Displays the inputbox to the user.
  #   Uses the choices and defaults given when the prompt was constructed.
  #   Returns an array of the values entered.
  #
  def show
    ret = inputbox( @prompts, @values, @choices, @title )
    @values = ret unless ret == false # Cancel
    return ret
  end

  #
  # [ Fixnum | String ]
  #
  #   returns the value associated with the given key or index
  #
  def []( key )
    return nil if key.nil?
    if key.is_a? Fixnum
      return @values[key]
    elsif key.is_a? String
      i = @prompts.index(key)
      return @values[i] unless i.nil?
    end
  end


  #
  # reset
  #
  #   Resets the values to the initial defaults
  #
  def reset
    @values = @defaults
  end

  #
  # each { |key, value| block }
  #
  #  Yield each prompt and value to the given block
  #
  def each
    # some kind of magic, heh?
    return unless block_given?
    prompts.each_with_index do |pr, i|
      yield pr, @values[i]
    end
  end

  #
  # prompts
  #   
  #   Returns an Array of prompts
  #
  def prompts
    @prompts
  end

  #
  # values
  #
  #  Returns an Array of input values
  #
  def values
    @values
  end

  #
  # save
  #
  #   Save the options to the registry for persistent storage
  #
  def save
    section = "Inputbox_" + @title
    i = "01"
    c = 0
    each do |k, v|
      Sketchup.write_default(section, "Prompt_#{i}", e(k.to_s))
      Sketchup.write_default(section, "Value_#{i}", e(v.to_s))
      if v = @choices[c]
	Sketchup.write_default(section, "Choice_#{i}", e(v.to_s))
      end
      if  v = @defaults[c]
	Sketchup.write_default(section, "Default_#{i}", e(v.to_s))
      end
      i.next!
      c += 1
    end
  end

  #
  # load
  #   
  #   Restores the Iinputbox from the registry.
  #
  def load
    section = "Inputbox_" + @title
    prompts = []
    values = []
    choices  = []
    defaults = []
    c = "01"
    while( prompt =  Sketchup.read_default(section, "Prompt_#{c}") )
      prompts << d(prompt)
      values << d(Sketchup.read_default(section, "Value_#{c}"))
      choices << d(Sketchup.read_default(section, "Choice_#{c}"))
      defaults << d(Sketchup.read_default(section, "Default_#{c}"))
      c.next!
    end
    @prompts = prompts
    @values = values
    @choices = choices
    @defaults = defaults
  end

  def to_hash
    h = {}
    each { |k, v|
      h[k] = v
    }
    return h
  end

  def set_size(w, h)
    @dlg.set_size(w, h) if @dlg
  end

  def show_dialog
    create_dialog.show_modal
    #@dlg.show_modal
  end

  private

  def to_html
    html = %(
    <html>
      <head>
	<title>#{@title}</title>
	<style> input.button {width: 6em;} </style>
	<script> function clicked(v) { window.location = 'skp:clicked@' + v; } </script>
	<script>

	function okClicked() {
	  var a = new Array();
	  var allPrompts = document.getElementsByName("_prompt");
          for(i=0; i<allPrompts.length; i++) {
            var e = allPrompts[i];
	    if (e.type == "text") {
	      a.push(e.id+"="+e.value);
	    } else if (e.type == "checkbox") {
	      if (e.checked) { a.push(e.id + "=" + 'true'); } else { a.push(e.id+"="+'false') }
	    } else if (e.type == "select-one") {
	      a.push(e.id + "=" + e.value);
	    }
	  } // for loop
	  window.location = 'skp:clicked@' + a.join('|');
          // alert(a.join(','));
	}
	</script>
    </head>
    <body>
    <table>)

    @prompts.each_with_index do |prompt, i|
      html << "<tr>\n"
      html << "  <td> #{prompt} </td>"
      html << "  <td> " # choices cell
      html << string_to_input(prompt, @values[i], @choices[i], @defaults[i])
      html << "  </td> " # choices cell
      html << "\n"
      html << "</tr>\n"
    end

    html << %(
      </table>
      <div style="text-align: center; margin-top: 1em;"> <input onclick="okClicked(#{IDOK});" style="width: 6em;"type="button" value="Ok">
      <input onclick="clicked(#{IDCANCEL});" style="width: 6em;" type="button" value="Cancel"></div>
      </body></html>)
    return html
  end


  def create_dialog
    @dlg = UI::WebDialog.new(@title)
    @dlg.set_html(to_html)
    @dlg.add_action_callback('clicked') { | d, a| dialog_button(d, a) }
    height = @prompts.length * 25 + 120
    @dlg.set_size(300, height)
    @dlg.set_on_close{ puts "Closed." }
    return @dlg
  end
    

  def dialog_button(d, a) # callback
    puts "dialog_button: #{a.inspect}"
    if a.to_i== IDCANCEL
      d.close
    else
      a.split('|').each do |pair|
	#p pair
	k, v = pair.split('=')
	puts "#{k} = #{v}"
	@values[@prompts.index(k)] = v
      end
      d.close
    end
  end

  def string_to_input(prompt, value, choice, default)
    html = ""
    value = value.to_s
    true_re  = Regexp.new("true|yes|y", 1)
    false_re = Regexp.new("false|no|n", 1)
    if choice.nil? # is selection?
      if default.class == TrueClass or default.class == FalseClass or value[true_re] or value[false_re]
	type  = "checkbox"
	if ((value.to_s)[true_re])
	  checked = "checked"
	else
	  checked = ""
	end
      else
	type = "text"
      end
      return %(<input name="_prompt" id="#{prompt}" type="#{type}" value="#{value}" #{checked} />)
    end
    choices = choice.split('|')
    html << "<select name=\"_prompt\" id=\"#{prompt}\">\n"
    choices.each do |choice|
      if choice == value
	selected = "selected"
      else
	selected = ""
      end
      html << %(    <option value="#{choice}" #{selected}>#{choice}</option>\n)
    end
    html << "    </select>\n"
    return html
  end


  #
  # These are helper methods for the add method and are
  # not available to the Inputbox user.
  #
  def add_prompt(prompt, default = nil)
    @prompts << prompt
    @values << default
    @defaults << default
    @choices << nil
  end

  def add_dropdown(prompt, choices, default = nil)
    @prompts << prompt
    @choices << choices.join("|")
    if default.nil?
      @values << choices[0]
      @defaults << choices[0]
    else
      @values << default
      @defaults <<  default
    end
  end

  #
  # e
  #
  #   Encode (escape) strings for writing to registry
  #
  def e(str)
    if str.is_a? String
      #str.gsub!(/%/, '\%')
      str.gsub!(/"/, '\"')
      #str.gsub!(/#/, '\#')
      str.gsub!(/\\/, '/')
      return str
      #s = [str].pack("a")
    end
  end

  #
  # d( string )
  #
  #   Decodes values read from Registry.
  #
  def d(s)
      return true if s == 'true'
      return false if s == 'false'
      return s
  end

end

